@@ -18,7 +18,8 @@
 
 DEFAULT_EXTENSION_SETTINGS = {
     mpris: {
-        enabled: true
+        enabled: true,
+        websiteSettings: {}
     },
     mprisMediaSessions: {
         enabled: true
@@ -28,6 +29,7 @@ DEFAULT_EXTENSION_SETTINGS = {
     },
     downloads: {
         enabled: true,
+        addToRecentDocuments: true,
         saveOriginUrl: false
     },
     tabsrunner: {
@@ -46,3 +48,8 @@ IS_FIREFOX = (typeof InstallTrigger !== "undefined"); // heh.
 
 // NOTE if you change this, make sure to adjust the error message shown in action_popup.html
 SUPPORTED_PLATFORMS = ["linux", "openbsd", "freebsd"];
+
+// Default MPRIS settings for websites
+const MPRIS_WEBSITE_SETTINGS = {
+    //"https://www.example.com": false
+};
