@@ -34,7 +34,12 @@
 Connection::Connection() :
     QObject()
 {
-    m_stdOut.open(STDOUT_FILENO, QIODevice::WriteOnly);
+    // Make really sure no one but us, who uses the correct format, prints to stdout
+    int newStdout = dup(STDOUT_FILENO);
+    // redirect it to stderr so it's not just swallowed
+    dup2(STDERR_FILENO, STDOUT_FILENO);
+    m_stdOut.open(newStdout, QIODevice::WriteOnly);
+
     m_stdIn.open(STDIN_FILENO, QIODevice::ReadOnly | QIODevice::Unbuffered);
 
     auto notifier = new QSocketNotifier(STDIN_FILENO, QSocketNotifier::Read, this);
@@ -68,11 +73,11 @@ void Connection::readData()
      * So we need to check for this condition ourselves
      * and exit. */
 
-    struct pollfd poll_stdin = {
-        .fd = STDIN_FILENO,
-        .events = POLLHUP,
-        .revents = 0
-    };
+    struct pollfd poll_stdin = {};
+    poll_stdin.fd = STDIN_FILENO;
+    poll_stdin.events = POLLHUP;
+    poll_stdin.revents = 0;
+
     if (poll (&poll_stdin, 1, 0) != 0) {
         // STDIN has HUP/ERR/NVAL condition
         qApp->exit(0);
@@ -88,7 +93,7 @@ void Connection::readData()
     }
 
     QByteArray data = m_stdIn.read(length);
-    if (data.length() != length) {
+    if (data.length() != int(length)) {
         m_stdIn.rollbackTransaction();
         return;
     }
