@@ -30,6 +30,7 @@
 struct EnvironmentDescription {
     QString applicationName;
     QString applicationDisplayName;
+    QString desktopFileName;
     QString organizationDomain;
     QString organizationName;
 };
@@ -59,6 +60,7 @@ public:
     Q_ENUM(Environment)
 
     void handleData(const QString &event, const QJsonObject &data) override;
+    QJsonObject handleData(int serial, const QString &event, const QJsonObject &data) override;
 
     Environment environment() const;
     QString environmentString() const; // dbus
