@@ -36,6 +36,8 @@ public:
     explicit TabsRunnerPlugin(QObject *parent);
     bool onLoad() override;
     bool onUnload() override;
+
+    using AbstractBrowserPlugin::handleData;
     void handleData(const QString &event, const QJsonObject &data) override;
 
     // dbus-exported
@@ -44,6 +46,6 @@ public:
     void SetMuted(int tabId, bool muted);
 
 private:
-    QDBusMessage m_tabsReplyMessage;
+    QVector<QDBusMessage> m_tabRequestMessages;
 
 };
